package ch5.imageio.plugins;

import java.io.IOException;
import java.awt.Rectangle;
import java.awt.image.Raster;
import javax.imageio.IIOImage;
import javax.imageio.ImageTypeSpecifier;
import javax.imageio.ImageWriter;
import javax.imageio.ImageWriteParam;
import javax.imageio.metadata.IIOMetadata;
import javax.imageio.metadata.IIOMetadataNode;
import javax.imageio.spi.ImageWriterSpi;
import javax.imageio.stream.ImageInputStream;
import javax.imageio.stream.ImageOutputStream;
import org.w3c.dom.*;

/**
 * ch5ImageWriter.java -- this class provides the functionality to
 * write an image of format ch5.  
 */
public class ch5v1ImageWriter extends ImageWriter {

    public ch5v1ImageWriter(ImageWriterSpi originatingProvider) {
	super(originatingProvider);
	streamMetadataWritten = false;
    }


    /**
     * this method returns null for now.  We will revisit it at the
     * end of this chapter after metadata has been discussed.
     */
    public IIOMetadata convertImageMetadata(IIOMetadata metadata,
					    ImageTypeSpecifier its,
					    ImageWriteParam param) {
	return null;
    }

    /**
     * this method returns null for now.  We will revisit it at the
     * end of this chapter after metadata has been discussed.
     */
    public IIOMetadata convertStreamMetadata(IIOMetadata metadata,
					     ImageWriteParam param) {
	return null;
    }

    /**
     * this method returns null for now.  We will revisit it at the
     * end of this chapter after metadata has been discussed.
     */
    public IIOMetadata getDefaultImageMetadata(ImageTypeSpecifier its,
					       ImageWriteParam param) {
	return null;
    }

    /**
     * this method returns null for now.  We will revisit it at the
     * end of this chapter after metadata has been discussed.
     */
    public IIOMetadata getDefaultStreamMetadata(ImageWriteParam param) {
	return null;
    }

    /**
     * write out the output image specified by index imageIndex using
     * the parameters specified by the ImageWriteParam object param
     */
    public void write(IIOMetadata metadata, 
		      IIOImage iioimage,
		      ImageWriteParam param) {
	Node root = null;
	Node dimensionsElementNode = null;

	if (iioimage.getRenderedImage() != null)
	    raster = iioimage.getRenderedImage().getData();
	else 
	    raster = iioimage.getRaster();

	/* since this format allows you to write multiple images,
	 * the streamMetadataWritten variable makes sure the 
	 * stream metadata is written only once.
	 * Not using metadata in this version, so using default value of
	 * 1 image per output stream.
	 */
	if (streamMetadataWritten == false) {
	    try {
		ios.writeBytes("5\n");
		ios.writeBytes("1"); 
		ios.flush();
	    }
	    catch (IOException ioe) {
		System.err.println("IOException "+ioe.getMessage());
	    }
	    streamMetadataWritten = true;
	}


	int sourceWidth = raster.getWidth();
	int sourceHeight = raster.getHeight();
	int destinationWidth = -1;
	int destinationHeight = -1;
	int sourceRegionWidth = -1;
	int sourceRegionHeight = -1;
	int sourceRegionXOffset = -1;
	int sourceRegionYOffset = -1;
	int xSubsamplingFactor = -1;
	int ySubsamplingFactor = -1;

        if (param == null)
            param = getDefaultWriteParam();

	/*
	 * get Rectangle object which will be used to clip
	 * the source image's dimensions.
	 */
	Rectangle sourceRegion = param.getSourceRegion();
	if (sourceRegion != null) {
	    sourceRegionWidth = (int)sourceRegion.getWidth();
	    sourceRegionHeight = (int)sourceRegion.getHeight();
	    sourceRegionXOffset = (int)sourceRegion.getX();
	    sourceRegionYOffset = (int)sourceRegion.getY();
		
	    /*
	     * correct for overextended source regions
	     */
	    if (sourceRegionXOffset + sourceRegionWidth > sourceWidth)
		destinationWidth = sourceWidth-sourceRegionXOffset;
	    else
		destinationWidth = sourceRegionWidth;

	    if (sourceRegionYOffset + sourceRegionHeight > sourceHeight)
		destinationHeight = sourceHeight-sourceRegionYOffset;
	    else
		destinationHeight = sourceRegionHeight;
	}
	else {
	    destinationWidth = sourceWidth;
	    destinationHeight = sourceHeight;
	    sourceRegionXOffset = sourceRegionYOffset = 0;
	}
	/*
	 * get subsampling factors 
	 */
	xSubsamplingFactor = param.getSourceXSubsampling();
	ySubsamplingFactor = param.getSourceYSubsampling();
	    
	destinationWidth = (destinationWidth-1)/xSubsamplingFactor + 1;
	destinationHeight = (destinationHeight-1)/ySubsamplingFactor + 1;

	byte[] sourceBuffer;
	byte[] destinationBuffer = new byte[destinationWidth];

        try {
	    ios.writeBytes(new String("\n"));
	    ios.writeBytes(new String(destinationWidth+ "\n"));
	    ios.writeBytes(new String(destinationHeight+ "\n"));

	    int jj;
	    int index;
	    for (int j=0; j<sourceWidth; j++) {
		sourceBuffer= (byte[])raster.getDataElements(0, j, sourceWidth, 1, null);
		jj = j - sourceRegionYOffset;
		if (jj % ySubsamplingFactor == 0) {
		    jj /= ySubsamplingFactor;
		    if ((jj >= 0) && (jj < destinationHeight)) {
			for (int i=0;i<destinationWidth;i++) {
			    index =  sourceRegionXOffset+i*xSubsamplingFactor;
			    destinationBuffer[i] = sourceBuffer[index];
			}
			ios.write(destinationBuffer, 0, destinationWidth);
			ios.flush();
		    }
		}
	    }
        }
        catch (IOException e) {
	    System.err.println("IOException: " + e.getMessage());
        }
    }

    public void setOutput(Object output) {
	super.setOutput(output);

        if (output == null) 
	    throw new IllegalArgumentException("output is null");

	if (!(output instanceof ImageOutputStream)) 
	    throw new IllegalArgumentException("output not an ImageOutputStream");

	ios =  (ImageOutputStream)output;
	streamMetadataWritten = false;
    }

    private ImageOutputStream ios;
    private boolean streamMetadataWritten;
    private Raster raster;
}
