package ch6;

import java.awt.*;
import javax.swing.*;
import java.io.*;
import javax.media.jai.JAI;
import javax.media.jai.operator.DFTDescriptor;
import javax.media.jai.KernelJAI;
import javax.media.jai.RenderedOp;
import javax.media.jai.RenderedImageList;
import java.awt.image.renderable.ParameterBlock;

/** 
    DFTTester -- displays the provided image along side its DFT
*/
public class DFTTester extends JFrame {

    public DFTTester(String filename) {
	setTitle("DFT Tester");
	RenderedOp inputImage = loadImage(filename);
	RenderedOp dftComplexImage = computeDFT(inputImage);
	RenderedOp dftMagnitudeImage = computeMagnitudes(dftComplexImage);
	RenderedOp dftLogImage = computeLogImage(dftMagnitudeImage);
	RenderedOp formattedLogImage = formatForDisplay(dftLogImage);

	getContentPane().setLayout(new GridLayout(1,2));
	getContentPane().add(new ch6Display(inputImage));
	getContentPane().add(new ch6Display(formattedLogImage));

	pack();
	show();
    }

    private RenderedOp loadImage(String filename) {
	ParameterBlock param = new ParameterBlock();
	param.add(filename);
	return JAI.create("Fileload", param);
    }

    /**
       perform Discrete Fourier Transform
    */
    private RenderedOp computeDFT(RenderedOp ro) {
	ParameterBlock param =  new ParameterBlock();
	param.addSource(ro);
	param.add(DFTDescriptor.SCALING_NONE);
	param.add(DFTDescriptor.REAL_TO_COMPLEX);
	return JAI.create("DFT", param);	
    }

    /**
       computes the magnitude image from a supplied complex image.  
       The number of bands will be decreased by 2
    */
    private RenderedOp computeMagnitudes(RenderedOp ro) {
	ParameterBlock param =  new ParameterBlock();
	param.addSource(ro);
	return JAI.create("Magnitude", param);
    }

    /**
       performs log(pixelValue+1)
    */
    private RenderedOp computeLogImage(RenderedOp ro) {
	ParameterBlock param;

	param =  new ParameterBlock();
	param.addSource(ro);
	double[] constant = {1.0};  
	param.add(constant);
	RenderedOp tmp = JAI.create("addConst", param);

	param =  new ParameterBlock();
	param.addSource(tmp);
	return JAI.create("log", param);
    }


    /**
       1. scales input image so that the maximum value is 255
       2. shifts scaled image so DC frequency is in center
       3. formats shifted image to a datatype of byte
    */
    private RenderedOp formatForDisplay(RenderedOp ro) {
	ParameterBlock param;

	param=  new ParameterBlock();
        param.addSource(ro);
        param.add(null);
        param.add(1);
        param.add(1);
        RenderedOp statsImage = JAI.create("Extrema", param);
	double[] maximum = (double[]) statsImage.getProperty("maximum");

	param = new ParameterBlock();
	param.addSource(ro);
	double[] scale = {255.0/maximum[0]};
	param.add(scale);
	double[] offset = {0.0};
	param.add(offset);
	RenderedOp rescaledImage = JAI.create("Rescale", param);	

	param = new ParameterBlock();
	param.addSource(rescaledImage);
	param.add(new Integer(rescaledImage.getWidth()/2));
	param.add(new Integer(rescaledImage.getHeight()/2));
	RenderedOp shiftedImage = JAI.create("PeriodicShift", param);	

	param = new ParameterBlock();
	param.addSource(shiftedImage);
	param.add(java.awt.image.DataBuffer.TYPE_BYTE);
	return JAI.create("format", param);	
    }

    public static void main(String[] args) {
	if (args.length != 1)
	    System.err.println("USAGE:  DFTTester filename");
	else
	    new DFTTester(args[0]);
    }
}
