package ch6;

import java.awt.*;
import java.awt.image.*;
import javax.swing.*;
import java.io.*;
import java.net.*;
import java.awt.image.renderable.ParameterBlock;
import javax.media.jai.RenderedOp;
import javax.media.jai.JAI;
import com.sun.media.jai.codec.SeekableStream;

/**
   ImageLoadTester -- loads an image using the JAI class' create method.
   This method uses 3 different operator names to load the image
   in 3 different manners, namely "stream", "fileload" and "url"

   It then prints out the properties of the loaded image
 */
public class ImageLoadTester extends JFrame {

    public ImageLoadTester(String filename) {
	SeekableStream stream = null;
	URL url = null;

	try {
	    url = new URL("file:"+filename);
	    stream = SeekableStream.wrapInputStream(url.openStream(), 
						    false);
	}
	catch (IOException ioe) {
	    System.err.println("IOException: " + ioe.getMessage());
	}

	params = new ParameterBlock();
	params.add(stream);
	RenderedOp streamImage = JAI.create("stream", params);

	params = new ParameterBlock();
	params.add(filename);
	RenderedOp fileImage = JAI.create("fileload", params);

	params = new ParameterBlock();
	params.add(url);
	RenderedOp urlImage = JAI.create("url", params);

	//Display Image Properties
 	String[] props = streamImage.getPropertyNames();
	for (int i=0;i<props.length;i++) {
	    System.out.print("Property: " + props[i] + ", ");
	    System.out.println(streamImage.getProperty(props[i]));
	}

	getContentPane().setLayout(new GridLayout(1, 3));
        getContentPane().add(new ch6Display(streamImage));
        getContentPane().add(new ch6Display(fileImage));
        getContentPane().add(new ch6Display(urlImage));

	pack();
	show();
    }

    public static void main(String[] args) {
       if (args.length != 1)
            System.err.println("USAGE:  ImageLoadTester filename");
        else
	    new ImageLoadTester(args[0]);
    }
    private ParameterBlock params;
}
