package ch6;

import java.awt.*;
import javax.swing.*;
import java.io.*;
import java.awt.image.*;
import java.awt.image.renderable.ParameterBlock;
import javax.media.jai.operator.CompositeDescriptor;
import javax.media.jai.JAI;
import javax.media.jai.PlanarImage;
import javax.media.jai.RenderedOp;
import javax.media.jai.RenderedImageList;

/**
   OtherPointOperatorsTester - demonstrates teh following operators:
   BandCombine, BandSelect, Composite and Piecewise
   Usage:
   OtherPointOperators fileName 0 //for BandCombine
   OtherPointOperators fileName 1 //for BandSelect
   OtherPointOperators fileName 2 //for Piecewise
   OtherPointOperators fileName1 fileName2 3 //for Composite
*/
public class OtherPointOperatorsTester extends JFrame {

    public OtherPointOperatorsTester(String fileName, int index) {
	PlanarImage img1 = loadImage(fileName);
	if (img1.getNumBands() < 3) {
	    System.err.print("Images must have 3 bands for these ");
	    System.err.println("demo methods to function as expected ");
	    System.exit(1);
	}
	    
	PlanarImage img2 = null;
	if (index == 0) {
	    setTitle("BandCombine Tester");
	    img2 = bandCombine(img1);
	}
	if (index == 1) {
	    setTitle("BandSelect Tester"); 
	    img2 = bandSelect(img1);
	}
	if (index == 2) {
	    setTitle("PieceWise Tester"); 
	    img2 = piecewise(img1);
	}
	getContentPane().setBackground(Color.white);
	getContentPane().setLayout(new GridLayout(1,2));
	getContentPane().add(new ch6Display(img1));
	getContentPane().add(new ch6Display(img2));
	pack();
	show();
    }

    public OtherPointOperatorsTester(String fileName1, 
				     String fileName2) {
	setTitle("Composite Tester");
	PlanarImage img1 = loadImage(fileName1);
	PlanarImage img2 = loadImage(fileName2);
	PlanarImage img3 = null;
	img3 = composite(img1, img2);

	getContentPane().setBackground(Color.white);
	getContentPane().setLayout(new GridLayout(1,3));
	getContentPane().add(new ch6Display(img1));
	getContentPane().add(new ch6Display(img2));
	getContentPane().add(new ch6Display(img3));
	pack();
	show();
    }

    private PlanarImage loadImage(String fileName) {
	ParameterBlock param =  new ParameterBlock();
	param.add(fileName);
	return JAI.create("fileload", param);
    }

    /**
       BandCombine operation in which the destination band components
       are;
       destinationRedComponent = 255 - sourceRedComponent 
       destinationGreenComponent = sourceBlueComponent;
       destinationBlueComponent = sourceGreenComponent;.

    */
    public PlanarImage bandCombine(PlanarImage pi) {
	double[][] matrix = {
	    { -1.0D,  0.0D, 0.0D, 255.0D },
	    { 0.0D,  0.0D, 1.0D, 0.0D },
	    { 0.0D,  1.0D, 0.0D, 0.0D },
	};
	ParameterBlock param =  new ParameterBlock();
	param.addSource(pi);
	param.add(matrix);
	return JAI.create("BandCombine", param);
    }

    /**
       if initial band order is red, green. blue
       destination band order will be red, blue, green
    */
    public PlanarImage bandSelect(PlanarImage pi) {
        int[] array = {0, 2, 1};

        ParameterBlock param =  new ParameterBlock();
        param.addSource(pi);
        param.add(array);
        return JAI.create("BandSelect", param);
    }



    /**
       performs compositing of PlanarImages pi1 and pi2 using
       normalized alpha values of .5 for pi1 and normalized alpha
       values of 1.0 (opaque) for pi2.  Thus, the destination pixels
       will be made up of equal parts of pi1 and pi2.
    */

    public PlanarImage composite(PlanarImage pi1, PlanarImage pi2) {
	byte alpha1Value = (byte)128;  //normalized value of .5
	byte alpha2Value = (byte)255;  //normalized value of 1.0

        ParameterBlock param =  new ParameterBlock();
        param.addSource(pi1);
        param.addSource(pi2);
        param.add(makeAlpha(pi1.getWidth(), pi1.getHeight(), 
			    alpha1Value, alpha1Value, alpha1Value));
        param.add(makeAlpha(pi2.getWidth(), pi2.getHeight(), 
			    alpha2Value, alpha2Value, alpha2Value));
        param.add(new Boolean(false));
        param.add(CompositeDescriptor.NO_DESTINATION_ALPHA);

        return JAI.create("Composite", param);
    }

    /**
       returns a PlanarImage containing 3 bands with samples being of
       type byte.  All sample in band0 will be set to alpha0, all
       samples in band1 will be set to alpha1 and all samples in band2
       will be set to alpha2.

    */
    private PlanarImage makeAlpha(float width, float height,
                                  byte alpha0, byte alpha1, byte alpha2) {

        Byte[] alphaValues;
        alphaValues = new Byte[3];
        alphaValues[0] = new Byte(alpha0); //alpha value for 1st band
        alphaValues[1] = new Byte(alpha1); //alpha value for 2nd band
        alphaValues[2] = new Byte(alpha2); //alpha value for 3rd band

        ParameterBlock param =  new ParameterBlock();
        param.add(width);
        param.add(height);
        param.add(alphaValues);
        RenderedOp ro = JAI.create("Constant", param);
        return ro;
    }


    /**
       performs piecewise linear mapping of a PlanarImage with 3 bands
       In this example: 
       all values under 50 will become 100
       all values over 200 will become 255
       all other values becomes linearly interpolated between the two, 
       i.e., 100 + (value-50)*(255-100)/(200-50)
    */
    public PlanarImage piecewise(PlanarImage pi) {
        float[][][] breakPoints = new float[3][2][2];
        breakPoints[0][0][0] = 50;
        breakPoints[0][1][0] = 100;
        breakPoints[0][0][1] = 200;
        breakPoints[0][1][1] = 255;

        breakPoints[1][0][0] = 50;
        breakPoints[1][1][0] = 100;
        breakPoints[1][0][1] = 200;
        breakPoints[1][1][1] = 255;

        breakPoints[2][0][0] = 50;
        breakPoints[2][1][0] = 100;
        breakPoints[2][0][1] = 200;
        breakPoints[2][1][1] = 255;

        ParameterBlock param =  new ParameterBlock();
        param.addSource(pi);
        param.add(breakPoints);

        return JAI.create("Piecewise", param);
    }

    public static void main(String[] args) {
	if (args.length == 2) {
	    try {
		int index = Integer.parseInt(args[1]);
		if (index < 3)
		    new OtherPointOperatorsTester(args[0], index);
	    }
	    catch (NumberFormatException nfe) {
		new OtherPointOperatorsTester(args[0], args[1]);
	    }
	}
	else {
	    System.err.println("USAGE:");

	    System.err.print("OtherPointOperators fileName 0 ");
	    System.err.println(" //for BandCombine");

	    System.err.print("OtherPointOperators fileName 1 ");
	    System.err.println(" //for BandSelect");

	    System.err.print("OtherPointOperators fileName 2 ");
	    System.err.println(" //for Piecewise");

	    System.err.print("OtherPointOperators fileName1 fileName2 ");
	    System.err.println(" //for Composite");
	}
    }
}
