package ch6;

import java.awt.*;
import javax.swing.*;
import java.awt.image.renderable.ParameterBlock;
import java.awt.image.RenderedImage;
import javax.media.jai.JAI;
import javax.media.jai.PlanarImage;
import javax.media.jai.RenderedOp;
import javax.media.jai.RenderableOp;

/**
   RenderableImageTester.java -- objects of this class 
   3.  create a RenderableOp by loading image contained in filename
   4.  create a 2nd RenderableOp representing the rotation 
       of the first RenderableOp
   5.  displays the final RenderedOp 
   6.  displays the RenderableOp according to desired image width and 
       height and rendering hints
*/
public class RenderableImageTester extends JFrame {
    public RenderableImageTester(String filename) {
	int xScale, yScale;

	RenderedOp imageSource = readInputFile(filename);
	RenderableOp renderableInput = getRenderable(imageSource);
	RenderableOp invertedRenderable;
	invertedRenderable = createRenderableInverted(renderableInput);
	RenderableOp rotatedRenderable;
	rotatedRenderable = createRenderableRotated(invertedRenderable);


	RenderingHints rh;
	rh = new RenderingHints(RenderingHints.KEY_RENDERING, 
				RenderingHints.VALUE_RENDER_SPEED);

	xScale = imageSource.getWidth()/2; 
	yScale = imageSource.getHeight()/2; 

	RenderedImage smallRendered;
	smallRendered = rotatedRenderable.createScaledRendering(xScale,
								yScale,
								rh);

	rh = new RenderingHints(RenderingHints.KEY_RENDERING, 
				RenderingHints.VALUE_RENDER_QUALITY);
	xScale = imageSource.getWidth()*2; 
	yScale = imageSource.getHeight()*2; 

	RenderedImage largeRendered;
	largeRendered = rotatedRenderable.createScaledRendering(xScale,
								yScale,
								rh);
	getContentPane().setLayout(new GridLayout(1,2));
	getContentPane().add(new ch6Display(smallRendered));
	getContentPane().add(new ch6Display(largeRendered));

	pack();
	show();
    }

    private RenderedOp readInputFile(String filename) {
	ParameterBlock pb = new ParameterBlock();
	pb.add(filename);
	return JAI.create("fileload", pb);
    }

    private RenderableOp getRenderable(RenderedOp ro) {
	ParameterBlock pb = new ParameterBlock();
	pb.addSource(ro);
	pb.add(null);
	pb.add(null);
	pb.add(null);
	pb.add(null);
	pb.add(null);
	return JAI.createRenderable("renderable", pb);
    }


    /**
       Returns a RenderableOp representing a inverted 
       version of RenderableOp toBeInverted
    */
    private RenderableOp createRenderableInverted(RenderableOp inputro) {
	ParameterBlock param;
	param =  new ParameterBlock();
	param.addSource(inputro);
	RenderableOp ro = JAI.createRenderable("Invert", param);

	return ro;
    }


    /**
       Returns a RenderableOp representing a rotated 
       version of RenderableOp toBeRotatedRO
    */
    private RenderableOp createRenderableRotated(RenderableOp inputro) {
	float angle = (float)((45.0/180.0)*Math.PI); //45 degree rotation

	ParameterBlock param;
	param =  new ParameterBlock();
	param.addSource(inputro);
	param.add(new Float(inputro.getWidth()/2));
	param.add(new Float(inputro.getHeight()/2));
	param.add(new Float(angle));
	RenderableOp ro = JAI.createRenderable("Rotate", param);

	return ro;
    }

    public static void main(String[] args) {
	if (args.length != 1)
	    System.err.println("Usage:  filename");
	else
	    new RenderableImageTester(args[0]);
    }
}
