package ch6;

import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import java.awt.image.renderable.ParameterBlock;
import java.awt.image.RenderedImage;
import javax.media.jai.JAI;
import javax.media.jai.PlanarImage;
import javax.media.jai.RenderedOp;

/**
   RenderingChangeEventTest.java -- objects of this class 
   1.  create a RenderedOp by loading filename1
   2.  create a 2nd RenderedOp representing rotation 
       of the first RenderedOp
   3.  renders and displays both RenderedOps
   4.  changes the input filename used in the 1st RenderedOp 
       to filename2 which generates a RenderingChangeEvent causing 
       the rotated image to change.
*/
public class RenderingChangeEventTest extends JFrame {

    public RenderingChangeEventTest(String filename1, String filename2) {
	this.filename2 = filename2;
	RenderedOp inputRO = JAI.create("fileload", filename1);
	RenderedOp rotatedRO = createRotatedImage(inputRO);

	/*
	  Force rotatedRO to be rendered.  This is not usually needed,
	  but for this example we need the rotatedRO object's rendering
	  to be done before the rotation angle is changed.
	*/
	RenderedImage tmp = (PlanarImage)rotatedRO.getRendering(); 

	// display original and rotated 
	getContentPane().setLayout(new GridLayout(1,2));
	getContentPane().add(new ch6Display(inputRO));
	getContentPane().add(new ch6Display(rotatedRO));
	pack();
	show();

	// wait 2 seconds so images don't change to quickly
	try{
	    Thread.sleep(2000);
	}
	catch(InterruptedException ie) {
	}

	changeFilename(rotatedRO);

	// redisplay images
	repaint();
    }

    /**
       Returns a RenderedOp representing a rotated 
       version of RenderedOp toBeRotatedRO
    */
    private RenderedOp createRotatedImage(RenderedOp toBeRotatedRO) {
	float angle = (float)((45.0/180.0)*Math.PI); //45 degree rotation

	ParameterBlock param;
	param =  new ParameterBlock();
	param.addSource(toBeRotatedRO);
	param.add(new Float(toBeRotatedRO.getWidth()/2));
	param.add(new Float(toBeRotatedRO.getHeight()/2));
	param.add(new Float(angle));
	RenderedOp ro = JAI.create("rotate", param);

	return ro;
    }

    /**
       1.  go to the RenderedOp's source image
       2.  change the filename parameter in its ParameterBlock
       3.  this will generate a RenderingChangeEvent which will 
       cause the rotatedRO RenderedOp to rerender its images
    */
    private void changeFilename(RenderedOp toBeChangedRO) {
	//get source RenderedOp
	RenderedOp tmpRO = (RenderedOp)toBeChangedRO.getSourceImage(0); 
	tmpRO.setParameter(filename2, 0);
    }

    public static void main(String[] args) {
	if (args.length != 2)
	    System.err.println("Usage:  filename1 filename2");
	else
	    new RenderingChangeEventTest(args[0], args[1]);
    }
    String filename2;
}
